@@ -1,15 +1,14 @@
 # ------------------------------------------------------------------
 # Warp Sequencer
-# (C) 2020 Michael DeHaan <michael@michaeldehaan.net> & contributors
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
 # Apache2 Licensed
 # ------------------------------------------------------------------
 
 # custom exception types for easier capture of specific errors
 
-class WarpException(Exception):
-    pass
+# TODO: verify we are using all of these and consolidate
 
-class AllClipsDone(WarpException):
+class WarpException(Exception):
     pass
 
 class NotFound(WarpException):
@@ -49,4 +48,16 @@ class TimeoutException(WarpException):
     pass
 
 class InvalidOpcode(WarpException):
-    pass
\ No newline at end of file
+    pass
+
+class StopException(WarpException):
+    pass
+
+class WebSlotCompilerException(WarpException):
+
+    def __init__(self, errors):
+        self.errors = errors
+
+    def __str__(self):
+        # TODO: this isn't surfaced anywhere, eliminate or surface them.
+        return "Web Slot Compile Failed: %s" % (self.errors)
