@@ -1,6 +1,6 @@
 # ------------------------------------------------------------------
 # Warp Sequencer
-# (C) 2020 Michael DeHaan <michael@michaeldehaan.net> & contributors
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
 # Apache2 Licensed
 # ------------------------------------------------------------------
 
@@ -35,6 +35,21 @@ class Devices(CollectionApi):
         return self._generic_add(name, locals())
 
     def auto_add_discovered(self):
-        for x in self.list_available():
+        available = self.list_available()
+
+        # add any devices from the computer config not in the present state
+        for x in available:
             if not self.lookup(x):
-                self.add(x)
\ No newline at end of file
+                self.add(x)
+
+        devices = [ x for x in self.song.devices.values() ]
+
+        # remove devices that are not currently attached to the computer
+        # this could happen if loading someone else's song file.
+        for device in devices:
+            if device.name not in available:
+                for instrument in self.song.instruments.values():
+                    if instrument.device == device:
+                        instrument.device = None
+                self.song.remove_device(device)
+
