@@ -1,6 +1,6 @@
 # ------------------------------------------------------------------
 # Warp Sequencer
-# (C) 2020 Michael DeHaan <michael@michaeldehaan.net> & contributors
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
 # Apache2 Licensed
 # ------------------------------------------------------------------
 
@@ -28,7 +28,7 @@ class Tracks(CollectionApi):
             instrument = self.api.instruments.lookup(instrument, require=True)
         return self._generic_add(name, locals())
 
-    def edit(self, name, id:str=None, new_name:str=None, instrument:str=None, instruments:list=None, instrument_mode:str=None, muted:bool=False):
+    def edit(self, name:str=None, id:str=None, new_name:str=None, instrument:str=None, instruments:list=None, instrument_mode:str=None, muted:bool=False):
         if instrument:
             instrument = self.api.instruments.lookup(instrument, require=True)
         if instruments is not None:
@@ -36,7 +36,10 @@ class Tracks(CollectionApi):
         return self._generic_edit(name, locals())
 
     def reorder(self, id=None, direction=None):
-        # FIXME: this is a mirror of what is in scenes.py
+
+        # TODO: this is a mirror of what is in scenes.py
+        # TODO: this is no longer exposed, we should implement this with drag methods similar to how scenes work
+        # TODO: can we remove this?
 
         object = self.lookup(id=id, require=True)
         assert object is not None
@@ -54,7 +57,14 @@ class Tracks(CollectionApi):
                 self.song.tracks[index] = other
                 self.song.tracks[index-1] = object
 
-    def delete(self, id=None):
-        # this object can't be referenced by any other objects so not to much to do here...
+    def delete(self, id=None, ignore_clips=False):
         obj = self.lookup(id=id, require=True)
+
+        if not ignore_clips:
+            clips = self.song.all_clips()
+            for c in clips:
+                if c.track == obj:
+                    self.api.player.stop_clips([c.name])
+                    self.song.remove_clip(scene=c.scene, track=c.track)
+
         return self._generic_remove(obj.name)
\ No newline at end of file
