@@ -1,11 +1,13 @@
 # ------------------------------------------------------------------
+# ------------------------------------------------------------------
 # Warp Sequencer
-# (C) 2020 Michael DeHaan <michael@michaeldehaan.net> & contributors
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
 # Apache2 Licensed
 # ------------------------------------------------------------------
 
 # this implements the public Python API for WarpSeq.
-# see examples/api/*.py for usage
+# see examples/api/*.py for usage. Direct access to model
+# objects through other means is not a "supported" or stable API contract.
 
 import gc
 import random
@@ -27,60 +29,67 @@ from warpseq.model.song import Song
 from warpseq.api.callbacks import Callbacks, DefaultCallbacks
 
 gc.disable()
+
 MIDI_PORTS = midi.get_devices()
 
 class Api(object):
 
     __slots__ = ( '_song', 'song', 'devices', 'instruments', 'scales',
                   'patterns', 'data_pools', 'transforms', 'scenes', 'tracks', 'clips',
-                  'player', 'song', '_callbacks' )
+                  'player', 'song', '_callbacks', 'web' )
+
+    def __init__(self, default_callbacks=True, web=False):
+
+        random.seed()
+        self.web = web
 
-    def __init__(self, default_callbacks=True):
-        self._reset()
         self._callbacks = Callbacks()
         if default_callbacks:
             self._callbacks.clear()
             self._callbacks.register(DefaultCallbacks())
 
+        self._song = Song(name='')
+
+        self.song = SongApi(self, self._song)
+
+        self.devices = Devices(self, self._song)
+        self.instruments = Instruments(self, self._song)
+        self.scales = Scales(self, self._song)
+        self.patterns = Patterns(self, self._song)
+        self.data_pools = DataPools(self, self._song)
+        self.transforms = Transforms(self, self._song)
+        self.scenes = Scenes(self, self._song)
+        self.tracks = Tracks(self, self._song)
+        self.clips = Clips(self, self._song)
+
+        self.player = Player(self, self._song)
+
+        if web:
+            self.extra_setup()
+
     def remove_callbacks(self):
         self._callbacks.clear()
 
     def add_callbacks(self, cb):
         self._callbacks.register(cb)
 
-    def _reset(self):
-
-        self._song = Song(name='')
-        self._setup_api()
-
-    def _setup_api(self):
-        self.song = SongApi(self, self._song)
-        self.devices = Devices(self, self._song)
-        self.instruments = Instruments(self, self._song)
-        self.scales  = Scales(self, self._song)
-        self.patterns = Patterns(self, self._song)
-        self.data_pools = DataPools(self, self._song)
-        self.transforms = Transforms(self, self._song)
-        self.scenes = Scenes(self, self._song)
-        self.tracks = Tracks(self, self._song)
-        self.clips = Clips(self, self._song)
-        self.player = Player(self, self._song)
+    def reset(self, init=True):
+        self.player.stop()
+        self._song.reset(clear=True)
+        if init and self.web:
+           self.extra_setup()
 
+    def extra_setup(self):
         random.seed()
-
-    # ------------------------------------------------------------------------------------------------------------------
-    # FIXME: song save/load work in progress
-
-    def reset(self):
-        self._reset()
-        self.scales.add(name='C-major', note='C', octave=0, scale_type='major')
+        self.scales.add(name='s1', note='C', octave=0, scale_type='major')
         for x in range(1,5):
             self.scenes.add(name='Scene %s' % x)
             self.tracks.add(name='Track %s' % x, instruments=[])
 
     def from_dict(self, data):
-        self._song = Song.from_dict(data)
-        self._setup_api()
+        self.reset(init=False)
+        self._song.load_from_dict(data)
+        self.devices.auto_add_discovered()
 
     def to_dict(self):
         return self._song.to_dict()
@@ -91,23 +100,3 @@ class Api(object):
 
     def to_json(self):
         return self._song.to_json()
-
-    def load(self, filename:str):
-        fh = open(filename, "r")
-        data = fh.read()
-        fh.close()
-        self.from_json(data)
-        self._song.filename = filename
-
-
-    def save(self):
-        if not self._song.filename:
-            raise InvalidUsage("no filename set, use save_as")
-        data = self._song.to_json()
-        fh = open(self._filename, "w+")
-        fh.write(data)
-        fh.close()
-
-    def save_as(self, filename:str):
-        self._song.filename = filename
-        self.save()
