@@ -1,6 +1,6 @@
 # ------------------------------------------------------------------
 # Warp Sequencer
-# (C) 2020 Michael DeHaan <michael@michaeldehaan.net> & contributors
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
 # Apache2 Licensed
 # ------------------------------------------------------------------
 
@@ -21,6 +21,12 @@ class BaseObject(object):
         assert length == 1
         return alist[0]
 
+    def is_hidden(self):
+        return False
+
+    def _additional_web_copy_steps(self, song, from_obj):
+        pass
+
 class NewReferenceObject(BaseObject):
 
     __slots__ = ( 'obj_id' )
@@ -38,6 +44,9 @@ class NewReferenceObject(BaseObject):
     def to_dict(self):
         return save_object(self)
 
+    def post_load(self):
+        pass
+
     @classmethod
     def from_dict(cls, song, data):
         return load_object(song, data)
