@@ -1,6 +1,6 @@
 # ------------------------------------------------------------------
 # Warp Sequencer
-# (C) 2020 Michael DeHaan <michael@michaeldehaan.net> & contributors
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
 # Apache2 Licensed
 # ------------------------------------------------------------------
 
@@ -37,7 +37,6 @@ CHORD_TYPES = dict(
 
 CHORD_TYPE_KEYS = set([x for x in CHORD_TYPES.keys()])
 
-
 class Chord(object):
 
     __slots__ = ( "notes", "root", "chord_type", "from_scale", "length", "deferred_expressions" )
@@ -52,10 +51,9 @@ class Chord(object):
             deferred_expressions = []
         self.deferred_expressions = deferred_expressions
 
-        # chord length is meaningless, this just makes some code not have to do a type check
+        # chord length is meaningless, this just makes some other code somewhere not have to do a type check
         self.length = None
 
-
         if self.notes is None:
             offsets = CHORD_TYPES[self.chord_type]
             notes = [root.copy()]
@@ -71,6 +69,10 @@ class Chord(object):
     def copy(self):
         return Chord(notes=self.notes, from_scale=self.from_scale)
 
+    def record_transform(self, transform):
+        for n in self.notes:
+            n.record_transform(transform)
+
     def with_notes(self, new_notes):
         return Chord(notes=new_notes, from_scale=self.from_scale)
 
@@ -151,7 +153,7 @@ class Chord(object):
 
     def record_octave_shift(self, value):
         for x in self.notes:
-            note.record_octave_shift(value)
+            x.record_octave_shift(value)
 
     def record_degree_shift(self, value):
         for x in self.notes:
