@@ -1,6 +1,6 @@
 # ------------------------------------------------------------------
 # Warp Sequencer
-# (C) 2020 Michael DeHaan <michael@michaeldehaan.net> & contributors
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
 # Apache2 Licensed
 # ------------------------------------------------------------------
 
@@ -12,14 +12,13 @@ from warpseq.playback.midi import midi_note_on, midi_note_off, midi_cc
 
 class Device(NewReferenceObject):
 
-    __slots__ = [ 'name', 'obj_id', '_midi_out', ]
-
-    SAVE_AS_REFERENCES = []
+    __slots__ = [ 'name', 'obj_id', '_midi_out', '_played_ts' ]
 
     def __init__(self, name=None, obj_id=None):
         self.name = name
         self.obj_id = obj_id
         self._midi_out = None
+        self._played_ts = None
         super(Device,self).__init__()
 
     def get_midi_out(self):
