@@ -1,4 +1,5 @@
 import random
+import time
 
 VARIABLES = dict()
 
@@ -14,17 +15,17 @@ class Evaluator(object):
 
     __slots__ = ()
 
-    SAVE_AS_REFERENCES = []
-
     def evaluate(self, context=None, subject=None):
         raise exceptions.NotImplementedError
 
+    def is_hidden(self):
+        # this is for the serializer which needs to exclude temporary scenes/tracks
+        return False
+
 class Probability(Evaluator):
 
     __slots__ = ('chance', 'a', 'b')
 
-    SAVE_AS_REFERENCES = []
-
     def __init__(self, chance, a, b):
         self.chance = chance
         self.a = a
@@ -43,8 +44,6 @@ class RandomChoice(Evaluator):
 
     __slots__ = ('values',)
 
-    SAVE_AS_REFERENCES = []
-
     def __init__(self, *values):
         self.values = values
         super(RandomChoice, self).__init__()
@@ -58,8 +57,6 @@ class RandomRange(Evaluator):
 
     __slots__ = ('low','high')
 
-    SAVE_AS_REFERENCES = []
-
     def __init__(self, low, high):
         self.low = low
         self.high = high
@@ -80,8 +77,6 @@ class RandomRange(Evaluator):
 
 class DataGrab(Evaluator):
 
-    SAVE_AS_REFERENCES = []
-
     __slots__ = ('pool',)
 
     def __init__(self, pool):
@@ -92,6 +87,7 @@ class DataGrab(Evaluator):
         pool = context.song.find_data_pool_by_name(self.pool)
         if pool is None:
             return None
+        pool._played_ts = time.perf_counter()
         data_slot = pool.get_next()
         value = data_slot.value
         res = evaluate(value, context, subject)
@@ -100,8 +96,6 @@ class DataGrab(Evaluator):
 
 class LoadVariable(Evaluator):
 
-    SAVE_AS_REFERENCES = []
-
     __slots__ = ('name',)
 
     def __init__(self, name):
@@ -115,8 +109,6 @@ class LoadVariable(Evaluator):
 
 class Negate(Evaluator):
 
-    SAVE_AS_REFERENCES = []
-
     __slots__ = ('what',)
 
     def __init__(self, what):
