@@ -1,6 +1,6 @@
 # ------------------------------------------------------------------
 # Warp Sequencer
-# (C) 2020 Michael DeHaan <michael@michaeldehaan.net> & contributors
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
 # Apache2 Licensed
 # ------------------------------------------------------------------
 
@@ -37,14 +37,15 @@ class Note(object):
 
     __slots__ = ( 'name', 'octave', 'tie', 'length', 'start_time', 'end_time', 'velocity', 'from_scale',
                   'from_parser', 'repeat', 'length_mod', 'tied', 'delay', 'no_shift', 'muted', 'track_copy',
-                  'ccs', 'deferred_expressions', 'from_context', 'octave_set_recorded', 'octave_shifts_recorded', 'degree_shifts_recorded')
+                  'ccs', 'deferred_expressions', '_from_context', 'octave_set_recorded', 'octave_shifts_recorded', 'degree_shifts_recorded',
+                  '_transforms_recorded')
 
     SAVE_AS_REFERENCES = []
 
     def __init__(self, name=None, octave=0, tie=False, length=None, start_time=None, end_time=None,
                  velocity=DEFAULT_VELOCITY, from_scale=None, from_parser=None, repeat=None, length_mod=1, tied=0, delay=0,
                  no_shift=False, muted=False, track_copy=None, ccs=None, deferred_expressions=None,
-                 from_context=None, octave_set_recorded=None, octave_shifts_recorded=0, degree_shifts_recorded=0):
+                 from_context=None, octave_set_recorded=None, octave_shifts_recorded=0, degree_shifts_recorded=0, transforms_recorded=None):
 
          if name in EQUIVALENCE_SET:
              name = NOTES[EQUIVALENCE.index(name)]
@@ -64,7 +65,7 @@ class Note(object):
          self.tied = tied
          self.delay = delay
          self.muted = muted
-         self.from_context = from_context
+         self._from_context = from_context
          # needed to process deferred track grabs
          self.octave_set_recorded = octave_set_recorded
          self.octave_shifts_recorded = octave_shifts_recorded
@@ -74,8 +75,12 @@ class Note(object):
              ccs = {}
          if deferred_expressions is None:
              deferred_expressions = []
+         if transforms_recorded is None:
+             transforms_recorded = []
+
          self.ccs = ccs
          self.deferred_expressions = deferred_expressions
+         self._transforms_recorded = transforms_recorded
 
          if track_copy is None:
              track_copy = []
@@ -112,7 +117,7 @@ class Note(object):
                     track_copy = self.track_copy[:],
                     ccs = self.ccs,
                     deferred_expressions = self.deferred_expressions[:],
-                    from_context = self.from_context,
+                    from_context = self._from_context,
                     octave_shifts_recorded = self.octave_shifts_recorded,
                     degree_shifts_recorded = self.degree_shifts_recorded,
                     octave_set_recorded = self.octave_set_recorded
@@ -148,6 +153,9 @@ class Note(object):
         # how many milliseconds should we add from ties marked after this note?
         return self.tied
 
+    def record_transform(self, transform):
+        self._transforms_recorded.append(transform)
+
     def record_octave_set(self, value):
         self.octave_set_recorded = value
         self.octave_shifts_recorded = 0
@@ -231,7 +239,7 @@ class Note(object):
         return n1
 
     def transpose(self, steps=0, semitones=0, degrees=0, octaves=0):
-        # FIXME: improve to not use the note table?
+        # TODO: improve to not use the note table?
 
         degree_steps = SCALE_DEGREES_TO_STEPS[str(degrees)]
         steps = steps + (semitones * 0.5) + degree_steps
@@ -250,6 +258,8 @@ class Note(object):
         return [ self ]
 
     def note_number(self):
+        if self.name is None:
+            return None
         return NOTES.index(self.name) + (12 * self.octave)
 
     def invert(self, *args):
@@ -261,7 +271,7 @@ class Note(object):
         return n1
 
     def __repr__(self):
-        # FIXME: this will be simplified once we are doing less note debugging
+        # TODO: this can be simplified now that we are doing less note debugging
         return "Note<%s|%s,len=%s,time=%s/%s,cc=%s,tie=%s,muted=%s,track_copy=%s,def=%s,tied=%s>" % (
             self.name,
             self.octave,
