@@ -1,15 +1,16 @@
 # ------------------------------------------------------------------
 # Warp Sequencer
-# (C) 2020 Michael DeHaan <michael@michaeldehaan.net> & contributors
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
 # Apache2 Licensed
 # ------------------------------------------------------------------
 
 from warpseq.model.evaluator import *
 from warpseq.model.note import Note
+from warpseq.model.chord import Chord
 from warpseq.model.registers import get_first_playing_note
 import warpseq.utils.serialization as serialization
-
-
+import traceback
+from warpseq.utils.cast import safe_float, safe_int, safe_bool
 
 def evaluate(context=None, subject=None, how=None):
     if isinstance(how, Evaluator):
@@ -36,8 +37,6 @@ class BaseSlot(object):
 
 class DataSlot(BaseSlot):
 
-    SAVE_AS_REFERENCES = []
-
     __slots__ = ('value',)
 
     def __init__(self, value=None):
@@ -45,15 +44,14 @@ class DataSlot(BaseSlot):
 
 class Slot(BaseSlot):
 
-    SAVE_AS_REFERENCES = []
-
     __slots__ = ( "note", "octave", "degree", "repeats", "length", "sharp", "flat", "chord_type", "inversion",
         "variables", "track_copy", "track_grab", "rest", "ccs", "delay", "velocity", "skip", "shuffle",
-        "reverse", "reset", "tie", "degree_shift", "octave_shift" )
+        "reverse", "reset", "tie", "degree_shift", "octave_shift", "D1", "D2", "D3", "D4", "D5", "D6", "D7", "D8", '_played_ts' )
 
     def __init__(self, note=None, octave=None, octave_shift=None, degree_shift=None, degree=None, repeats=None, length=None, sharp=None, flat=None,
                  chord_type=None, inversion=None, variables=None, track_copy=None, track_grab=None, ccs=None,
-                 delay=None, velocity=None, skip=None, shuffle=None, reverse=None, reset=None, tie=None, rest=None):
+                 delay=None, velocity=None, skip=None, shuffle=None, reverse=None, reset=None, tie=None, rest=None, D1=None,
+                 D2=None, D3=None, D4=None, D5=None, D6=None, D7=None, D8=None):
 
         self.note = note
         self.octave = octave
@@ -80,6 +78,15 @@ class Slot(BaseSlot):
         self.degree_shift = degree_shift
         self.octave_shift = octave_shift
 
+        self.D1 = D1
+        self.D2 = D2
+        self.D3 = D3
+        self.D4 = D4
+        self.D5 = D5
+        self.D6 = D6
+        self.D7 = D7
+        self.D8 = D8
+
     def _standardize_input(self, note):
 
         from warpseq.model.note import Note
@@ -99,29 +106,40 @@ class Slot(BaseSlot):
 
     # ==================================================================================================================
 
-    def evaluate(self, context, obj):
+    def is_hidden(self):
+        # needed to appease the serializer
+        return False
 
+    # ==================================================================================================================
+
+    def evaluate(self, context, obj):
         # evaluates a Note or Chord or None and always returns a Chord or None
 
         from warpseq.model.note import Note
         from warpseq.model.chord import Chord
 
-        result = self._compute(context, obj)
+        try:
+            result = self._compute(context, obj)
+        except:
+            traceback.print_exc()
+            result = obj
 
         if not result:
             return None
         elif type(result) == Note:
             return Chord(notes=[result])
+
+        if type(result) == Chord:
+            for r in result.notes:
+                r._from_context = context
+
         return result
 
     # ==================================================================================================================
 
     def _compute(self, context, note):
 
-        # FIXME: rename 'note' to 'obj'
-
-        #print("-------")
-
+        # TODO: rename 'note' to 'obj' - could be a chord or note.
         note = self._standardize_input(note).copy()
 
         if note.length is None:
@@ -161,45 +179,75 @@ class Slot(BaseSlot):
         # NOTE BASICS HERE
 
         if self.note:
+            # this isn't really used by the Web UI but is used by some API examples
             note = note.with_name(evaluate(context=context, subject=note, how=self.note))
-            assert note.length is not None
+
+        d1 = evaluate(context=context, subject=note, how=self.D1)
+        d2 = evaluate(context=context, subject=note, how=self.D2)
+        d3 = evaluate(context=context, subject=note, how=self.D3)
+        d4 = evaluate(context=context, subject=note, how=self.D4)
+        d5 = evaluate(context=context, subject=note, how=self.D5)
+        d6 = evaluate(context=context, subject=note, how=self.D6)
+        d7 = evaluate(context=context, subject=note, how=self.D7)
+        d8 = evaluate(context=context, subject=note, how=self.D8)
+        drums = [ d1, d2, d3, d4, d5, d6, d7, d8 ]
+        ok_drums = [ x for x in drums if x ]
+        drum_inst = context.pattern.drum_config
+
+        if drum_inst and len(ok_drums):
+            notes = []
+            if d1:
+                notes.append(Note(name=drum_inst.drum1_note, octave=int(drum_inst.drum1_octave)))
+            if d2:
+                notes.append(Note(name=drum_inst.drum2_note, octave=int(drum_inst.drum2_octave)))
+            if d3:
+                notes.append(Note(name=drum_inst.drum3_note, octave=int(drum_inst.drum3_octave)))
+            if d4:
+                notes.append(Note(name=drum_inst.drum4_note, octave=int(drum_inst.drum4_octave)))
+            if d5:
+                notes.append(Note(name=drum_inst.drum5_note, octave=int(drum_inst.drum5_octave)))
+            if d6:
+                notes.append(Note(name=drum_inst.drum6_note, octave=int(drum_inst.drum6_octave)))
+            if d7:
+                notes.append(Note(name=drum_inst.drum7_note, octave=int(drum_inst.drum7_octave)))
+            if d8:
+                notes.append(Note(name=drum_inst.drum8_note, octave=int(drum_inst.drum8_octave)))
+            if len(notes):
+                note = Chord(notes=notes)
+
 
         if self.degree:
-            res = evaluate(context=context, subject=note, how=self.degree) - 1
-            # print("debug: set scale scale degree: %s" % res)
+            res = safe_int(evaluate(context=context, subject=note, how=self.degree)) - 1
             note = note.scale_transpose(context.scale,  res)
 
         if self.octave is not None:
-            res = evaluate(context=context, subject=note, how=self.octave)
+            res = safe_int(evaluate(context=context, subject=note, how=self.octave))
             note = note.with_octave(res)
-            #print("debug: set octave=%s" % res)
             note.record_octave_set(res)
 
         # -----
         # NOTE MODIFICATIONS HERE
 
         if self.octave_shift is not None:
-            how = evaluate(context=context, subject=note, how=self.octave_shift)
+            how = safe_int(evaluate(context=context, subject=note, how=self.octave_shift))
             note = note.transpose(octaves=how)
-            #print("octave transpose: %s" % how)
             note.record_octave_shift(how)
 
         if self.degree_shift is not None:
-            how = evaluate(context=context, subject=note, how=self.degree_shift)
+            how = safe_int(evaluate(context=context, subject=note, how=self.degree_shift))
             note = note.scale_transpose(context.scale, how)
-            #print("degree shift: %s" % how)
             note.record_degree_shift(how)
 
         if self.repeats:
-            note = note.with_repeat(evaluate(context=context, subject=note, how=self.repeats))
+            note = note.with_repeat(safe_int(evaluate(context=context, subject=note, how=self.repeats)))
 
         if self.length and self.length != 1:
             note = note.with_length_mod(evaluate(context=context, subject=note, how=self.length))
 
-        if self.sharp and evaluate(context=context, subject=note, how=self.sharp):
+        if self.sharp and safe_bool(evaluate(context=context, subject=note, how=self.sharp)):
             note = note.transpose(semitones=1)
 
-        if self.flat and evaluate(context=context, subject=note, how=self.flat):
+        if self.flat and safe_bool(evaluate(context=context, subject=note, how=self.flat)):
             note = note.transpose(semitones=-1)
 
         if self.chord_type:
@@ -225,18 +273,18 @@ class Slot(BaseSlot):
             items = {}
             for (k,v) in self.ccs.items():
                 items = {}
-                items[k] = evaluate(context=context, subject=note, how=v)
+                items[k] = safe_int(evaluate(context=context, subject=note, how=v))
             for (k,v) in items.items():
                 note = note.with_cc(k,v)
 
         if self.delay:
-            note = note.with_delay(evaluate(context=context, subject=note, how=self.delay))
+            note = note.with_delay(safe_float(evaluate(context=context, subject=note, how=self.delay)))
 
         if self.velocity is not None:
-            note = note.with_velocity(evaluate(context=context, subject=note, how=self.velocity))
+            note = note.with_velocity(safe_int(evaluate(context=context, subject=note, how=self.velocity)))
 
         if self.inversion:
-            note = note.invert(evaluate(context=context, subject=note, how=self.inversion))
+            note = note.invert(safe_int(evaluate(context=context, subject=note, how=self.inversion)))
 
         return note
 
@@ -251,7 +299,6 @@ def defer_track_grab(context=None, track=None, subject=None, how=None):
         return None
     copy_tracks = playing.track_copy
     if copy_tracks and track not in copy_tracks:
-        #print("on track (%s), this was not for me" % (track.name))
         return None
 
     new_note = subject.replace_with_note(playing.name, playing.octave)
@@ -261,14 +308,13 @@ def defer_track_grab(context=None, track=None, subject=None, how=None):
 
     first = subject
     if first.octave_set_recorded is not None:
-        new_note = subject.with_octave(first.octave_set_recorded)
+        new_note = new_note.with_octave(first.octave_set_recorded)
 
-    new_note = subject.transpose(octaves=first.octave_shifts_recorded)
+    new_note = new_note.transpose(octaves=first.octave_shifts_recorded)
 
     if playing.from_scale:
-        new_note = subject.scale_transpose(playing.from_scale, first.degree_shifts_recorded)
+        new_note = new_note.scale_transpose(playing.from_scale, first.degree_shifts_recorded)
 
-    #print("on track (%s), grabbed (%s)" % (track.name, new_note))
     return new_note
 
 
