@@ -1,6 +1,6 @@
 # ------------------------------------------------------------------
 # Warp Sequencer
-# (C) 2020 Michael DeHaan <michael@michaeldehaan.net> & contributors
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
 # Apache2 Licensed
 # ------------------------------------------------------------------
 
@@ -10,18 +10,31 @@
 import json
 
 from warpseq.model.base import NewReferenceObject
+from warpseq.model.status import status_report
 
 FORMAT_VERSION = 0.1
 DEFAULT_NAME = 'Warp Song'
 
+TRACK_HEADER_TEMPLATE = """
+<div class="ag-cell-label-container" role="presentation">
+<span ref="eMenu" class="ag-header-icon ag-header-cell-menu-button"></span>
+<div ref="eLabel" class="ag-header-cell-label" role="presentation">
+<span ref="eSortOrder" class="ag-header-icon ag-sort-order" ></span>
+<span ref="eSortAsc" class="ag-header-icon ag-sort-ascending-icon" ></span>
+<span ref="eSortDesc" class="ag-header-icon ag-sort-descending-icon" ></span>
+<span ref="eSortNone" class="ag-header-icon ag-sort-none-icon" ></span>
+<a href="#" onclick="load_into_nav(TRACK);load_track_item('%s')"> <span ref="eText" class="ag-header-cell-text" role="columnheader"></span></a>
+<span ref="eFilter" class="ag-header-icon ag-filter-icon"></span>
+</div>
+</div>
+"""
+
+SAVE_FIRST = [ 'devices', 'instruments', 'scales', 'data_pools', 'patterns', 'transforms', 'scenes', 'tracks', 'clips' ]
+
 class Song(NewReferenceObject):
 
-    # FIXME: warning slot order might matter as it is used by the serializer
-
     __slots__ = [ 'name',  'devices', 'instruments', 'tracks', 'clips', 'scale', 'tempo',  'scales', 'scenes',
-                  'transforms', 'patterns', 'data_pools', 'obj_id', 'filename' ]
-
-    SAVE_AS_REFERENCES = []
+                  'transforms', 'patterns', 'data_pools', 'obj_id', 'filename', '_SAVE_FIRST' ]
 
     def __init__(self, name='Warp Song', clips = None, scale=None, tempo=120, devices=None, instruments=None, scales=None,
                  scenes=None, tracks=None, transforms=None, patterns=None, data_pools=None, obj_id=None, filename='warp.json'):
@@ -41,10 +54,15 @@ class Song(NewReferenceObject):
         self.obj_id = obj_id
         self.filename = filename
 
+        self._SAVE_FIRST = SAVE_FIRST
+
         super(Song, self).__init__()
 
         self.reset(clear=False)
 
+    def status_report(self):
+        return status_report(self)
+
     def details(self):
         # return public fields - this is a bit different from other classes as this is the top level object
         results =  dict(
@@ -103,7 +121,6 @@ class Song(NewReferenceObject):
         """
         Returns the scene with the given object ID.
         """
-        print("SCENES: %s" % self.scenes)
         matching = [ x for x in self.scenes if str(x.obj_id) == str(obj_id) ]
         if len(matching) == 0:
             return None
@@ -117,11 +134,6 @@ class Song(NewReferenceObject):
         if len(matching) == 0:
             return None
 
-        #print("FOUND A TRACK")
-        #print("*********")
-        #print("INST=%s" % matching[0].instruments)
-        #raise Exception("STOP!")
-
         return matching[0]
 
     def find_clip(self, obj_id):
@@ -167,6 +179,18 @@ class Song(NewReferenceObject):
                 return v
         return None
 
+    def find_scale_by_name(self, name):
+        for (k,v) in self.scales.items():
+            if v.name == name:
+                return v
+        return None
+
+    def find_transform_by_name(self, name):
+        for (k,v) in self.transforms.items():
+            if v.name == name:
+                return v
+        return None
+
     def find_data_pool_by_name(self, name):
         for (k,v) in self.data_pools.items():
             if v.name == name:
@@ -175,14 +199,12 @@ class Song(NewReferenceObject):
 
     def to_dict(self):
         """
-        Returns the data for the entire song file.
-        This can be reversed with from_dict.
+        Returns the data for the entire song file. This can be reversed with from_dict.
         """
         global FORMAT_VERSION
         data = super().to_dict()
         data['FORMAT_VERSION'] = FORMAT_VERSION
         data['OBJ_COUNTER'] = NewReferenceObject.new_object_id()
-        #print("SONG=%s" % json.dumps(data, sort_keys=True, indent=4))
         return data
 
     def to_json(self):
@@ -194,8 +216,7 @@ class Song(NewReferenceObject):
     @classmethod
     def from_dict(cls, data):
 
-
-        # FIXME: TODO: we should be able to use the deserializer here too?
+        # TODO: we should be able to use the deserializer here also? If not, explain why.
 
         """
         Loads a song from a dictionary.
@@ -208,8 +229,8 @@ class Song(NewReferenceObject):
 
         format_version = data.get('FORMAT_VERSION', 0)
         if format_version > FORMAT_VERSION:
-            # FIXME:  change exception type
-            print("can not open data from a newer version of this program")
+            # TODO: raise exception here?
+            print("warning: likely cannot open data from a newer version of this program")
 
         COUNTER = data['OBJ_COUNTER']
 
@@ -226,6 +247,7 @@ class Song(NewReferenceObject):
         from . data_pool import DataPool
 
         song.scales = { str(k) : Scale.from_dict(song, v) for (k,v) in data['scales'].items() }
+        # BOOKMARK
         song.devices = { str(k) : Device.from_dict(song, v) for (k,v) in data['devices'].items() }
         song.instruments =  { str(k) : Instrument.from_dict(song, v) for (k,v) in data['instruments'].items() }
 
@@ -243,10 +265,41 @@ class Song(NewReferenceObject):
 
         return song
 
+    def load_from_dict(self, data):
+
+        loaded = Song.from_dict(data)
+
+        self.name = loaded.name
+        self.filename = loaded.filename
+        self.tempo = loaded.tempo
+
+        self.scales = loaded.scales
+        self.devices = loaded.devices
+        self.instruments = loaded.instruments
+        self.scenes = loaded.scenes
+        self.tracks = loaded.tracks
+
+        self.transforms = loaded.transforms
+        self.patterns = loaded.patterns
+        self.data_pools = loaded.data_pools
+
+        self.clips = loaded.clips
+        self.scale = loaded.scale
+        self.tempo = loaded.tempo
+        self.obj_id = loaded.obj_id
+
+    def all_clips(self):
+        return [ x for x in self.clips.values() ]
+
+    def all_patterns(self):
+        return [ x for x in self.patterns.values() ]
+
+    def all_data_pools(self):
+        return [ x for x in self.data_pools.values() ]
+
     def next_scene(self, scene):
         """
         Returns the scene that is positioned after this one in the song.
-        This uses the scene array, implying (FIMXE) we need a method to reorder scenes.
         """
         index = None
         for (i,x) in enumerate(self.scenes):
@@ -294,17 +347,12 @@ class Song(NewReferenceObject):
         clip.track = track
         clip.scene = scene
 
-        #assert clip.scene is not None
-
         track.add_clip(clip)
         scene.add_clip(clip)
 
-        #assert clip.track is not None
-        #assert clip.scene is not None
-
         return clip
 
-    def remove_clip(self, scene=None, track=None):
+    def remove_clip(self, scene=None, track=None, clip=None):
         """
         Deletes a clip.  The name isn't used - specify the scene and track.
         """
@@ -312,13 +360,17 @@ class Song(NewReferenceObject):
         # removing a clip returns a clip object that can be used with *assign* to add the
         # clip back.
 
-        #assert scene is not None
-        #assert track is not None
+        if not clip:
+            clip = self.get_clip_for_scene_and_track(scene=scene, track=track)
 
-        clip = self.get_clip_for_scene_and_track(scene=scene, track=track)
         if clip is None:
             return None
 
+        if not track:
+            track = clip.track
+        if not scene:
+            scene = clip.scene
+
         track.remove_clip(clip)
         scene.remove_clip(clip)
 
@@ -339,8 +391,6 @@ class Song(NewReferenceObject):
         """
         Returns the clip at the intersection of the scene and track.
         """
-        #assert scene is not None
-        #assert track is not None
         results = []
         clips = scene.clips(self)
         for clip in clips:
@@ -441,7 +491,6 @@ class Song(NewReferenceObject):
         """
         Adds some transform objects to the song.
         """
-        #assert type(transforms) == list
         for x in transforms:
             self.transforms[str(x.obj_id)] = x
 
@@ -450,3 +499,61 @@ class Song(NewReferenceObject):
         Removes a transform object from the song.
         """
         del self.transforms[str(transform.obj_id)]
+
+    def get_web_grid(self):
+
+        # this returns grid info for UI display
+
+        columns = []
+
+        columns.append(dict(
+           headerName="Scene",
+           field="scene",
+           editable=False,
+           rowDrag=True,
+           rowDragText="Scene"
+        ))
+
+        for t in self.tracks:
+            columns.append(dict(
+                headerName=t.name,
+                field=t.name,
+                editable=False,
+                headerComponentParams=(dict(template=TRACK_HEADER_TEMPLATE % t.obj_id)),
+            ))
+
+        rows = []
+        for s in self.scenes:
+            row = dict()
+            row['scene'] = dict(type='scene', scene_id=s.obj_id, scene_name=s.name)
+            for t in self.tracks:
+                clip = self.get_clip_for_scene_and_track(scene=s, track=t)
+                if clip:
+                    row[t.name] = dict(type='clip', track_id=t.obj_id, scene_id=s.obj_id, clip_id=clip.obj_id, clip_name=clip.name)
+                else:
+                    row[t.name] = dict(type='empty_cell', track_id=t.obj_id, scene_id=s.obj_id)
+            rows.append(row)
+
+        result = dict(
+            column_defs = columns,
+            row_data = rows
+        )
+
+        return result
+
+    def update_scene_order_for_ui(self, data):
+
+        scene_ids = [ x["scene"]["scene_id"] for x in data ]
+        scenes = [ self.find_scene(x) for x in scene_ids ]
+        self.scenes = scenes
+        return True
+
+    def delete_web_rows(self, data):
+
+        results = []
+        for (i,x) in enumerate(self.scenes):
+            if i not in data:
+                results.append(x)
+        self.scenes = results
+        return True
+
