@@ -1,6 +1,6 @@
 # ------------------------------------------------------------------
 # Warp Sequencer
-# (C) 2020 Michael DeHaan <michael@michaeldehaan.net> & contributors
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
 # Apache2 Licensed
 # ------------------------------------------------------------------
 
@@ -19,23 +19,22 @@ INSTRUMENT_MODE_CHOICES = [ ALL_INSTRUMENTS, ROTATE, ROTATE_CHORDS, SPREAD ]
 
 class Track(NewReferenceObject):
 
-    __slots__ = [ 'name', 'muted', 'instruments', 'clip_ids', 'obj_id', 'instrument_mode', '_all_instruments', '_hocket_roller' ]
+    __slots__ = [ 'name', 'muted', 'instruments', 'clip_ids', 'obj_id', 'instrument_mode', '_all_instruments', '_hocket_roller', 'hidden', '_played_ts' ]
 
-    SAVE_AS_REFERENCES = [ 'instruments' ]
-
-    def __init__(self, name=None, muted=False, instrument=None, instruments=None, instrument_mode=ALL_INSTRUMENTS, clip_ids=None, obj_id=None):
+    def __init__(self, name=None, muted=False, instrument=None, instruments=None, instrument_mode=ALL_INSTRUMENTS, clip_ids=None, obj_id=None, hidden=None):
 
         self.name = name
         self.muted = muted
+        self.hidden = hidden
         self.obj_id = obj_id
 
-        if not instruments and instrument:
-            instruments = [ instrument ]
-        if instruments is None:
-            instruments = []
-
         self.instruments = instruments
 
+        if not instruments and instrument:
+            self.instruments = [ instrument ]
+        if instruments is None:
+            self.instruments = []
+
         if instrument_mode is None:
             instrument_mode = ALL_INSTRUMENTS
 
@@ -49,12 +48,15 @@ class Track(NewReferenceObject):
 
         self._hocket_roller = utils.roller(self.instruments)
 
+        self._played_ts = None
+
         super(Track, self).__init__()
 
 
     # ------------------------------------------------------------------------------------------------------------------
 
     def get_instruments(self, evt, chosen, instrument_mode):
+
         if instrument_mode == ALL_INSTRUMENTS:
             return self.instruments
         elif instrument_mode in [ROTATE_CHORDS, SPREAD]:
@@ -86,3 +88,5 @@ class Track(NewReferenceObject):
     def remove_clip(self, clip):
         self.clip_ids = [ c for c in self.clip_ids if c != clip.obj_id ]
 
+    def is_hidden(self):
+        return self.hidden
\ No newline at end of file
