@@ -1,6 +1,6 @@
 # ------------------------------------------------------------------
 # Warp Sequencer
-# (C) 2020 Michael DeHaan <michael@michaeldehaan.net> & contributors
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
 # Apache2 Licensed
 # ------------------------------------------------------------------
 
@@ -13,6 +13,8 @@ from .directions import *
 from warpseq.model.context import Context
 from warpseq.model.slot import Slot
 import sys
+from warpseq.api.exceptions import WebSlotCompilerException
+from warpseq.parser.web_slots import WebSlotCompiler
 
 CHORDS = 'chords'
 NOTES = 'notes'
@@ -21,21 +23,27 @@ APPLIES_CHOICES = [ CHORDS, NOTES, BOTH ]
 
 class Transform(NewReferenceObject, Directionable):
 
-    __slots__ = [ 'name', 'slots', '_current_slots', 'divide', 'applies_to', 'obj_id',
-                  'direction', '_current_direction', '_iterator', '_slot_mods', 'auto_reset' ]
-
-    SAVE_AS_REFERENCES = [ ]
+    __slots__ = [ 'name', 'slots', 'arp', '_current_slots', 'divide', 'applies_to', 'obj_id',
+                  'direction', '_current_direction', '_iterator', '_slot_mods', 'auto_reset', 'audition_instrument',
+                  'audition_pattern', 'web_slots', '_played_ts']
 
     # ==================================================================================================================
 
-    def __init__(self, name=None, slots=None, divide=1, obj_id=None,
-                 applies_to=BOTH, direction=FORWARD, auto_reset=False):
+    def __init__(self, name=None, slots=None, divide=None, obj_id=None,
+                 applies_to=BOTH, direction=FORWARD, auto_reset=False, arp=True, audition_instrument=None,
+                 audition_pattern=None, web_slots=None):
+
+        if not slots and not web_slots:
+            web_slots = [ dict() for x in range(0,5) ]
+
+        self.slots = slots
+        self.set_slots_from_web_slots(web_slots)
+
+        # we'll add in web_slots compile soon
+        if self.slots is None:
+            self.slots = []
 
         self.name = name
-        self.slots = slots
-
-        for x in slots:
-            assert isinstance(x, Slot)
 
         self.divide = divide
         self.applies_to = applies_to
@@ -45,8 +53,14 @@ class Transform(NewReferenceObject, Directionable):
         self._current_direction = direction
         self.auto_reset = auto_reset
         self.applies_to = applies_to
+        self.audition_instrument = audition_instrument
+        self.audition_pattern = audition_pattern
+        self.arp = arp
 
         assert applies_to in APPLIES_CHOICES
+
+        self._played_ts = None
+
         self.reset()
 
         super(Transform, self).__init__()
@@ -106,6 +120,9 @@ class Transform(NewReferenceObject, Directionable):
             (process, arpeggiate) = self._get_should_process_and_arpeggiate(chord)
             divide = self._get_effective_divide(chord)
 
+            if not self.arp:
+                arpeggiate = False
+
             notes_iterator = self._get_notes_iterator(chord, arpeggiate)
 
             original_start = chord.notes[0].start_time
@@ -138,3 +155,41 @@ class Transform(NewReferenceObject, Directionable):
 
         return results
 
+    # ===================================================================================================================
+
+    def set_slots_from_web_slots(self, web_slots):
+         # TODO: lots of duplication with pattern.py in these, make a common base class
+
+         """
+         When deserializing web slots, run the compiler to generate the actual slot objects the engine can run on.
+         """
+
+         ok = True
+         errors = None
+         if web_slots is not None:
+             (ok, slots, errors) =  WebSlotCompiler(for_transform=True, pattern_type=None).compile(web_slots)
+             if not ok:
+                 return
+             else:
+                 self.slots = slots
+         else:
+             if self.slots is None:
+                 self.slots = []
+         self.web_slots = web_slots
+         if not ok:
+             raise WebSlotCompilerException(errors)
+
+    def get_web_slot_grid_for_ui(self, category):
+        return WebSlotCompiler(for_transform=True, pattern_type=None).get_grid(self.web_slots, category)
+
+    def update_web_slots_for_ui(self, data):
+        new_web_slots = WebSlotCompiler(for_transform=True, pattern_type=None).update_web_slots_from_ui(self.web_slots, data)
+        self.set_slots_from_web_slots(new_web_slots)
+
+    def delete_web_slot_rows(self, data):
+        # TODO: copied from pattern, reduce duplication
+        new_web_slots = []
+        for (i,x) in enumerate(self.web_slots):
+            if i not in data:
+                new_web_slots.append(x)
+        self.set_slots_from_web_slots(new_web_slots)
\ No newline at end of file
