@@ -1,3 +1,9 @@
+# ------------------------------------------------------
+# Warp Sequencer
+# (C) 2020 Michael DeHaan <michael@michaeldehaan.net>
+# Apache2 Licensed
+# -----------------------------------------------------
+
 import json
 
 class BasePacket(object):
@@ -37,13 +43,14 @@ class CommandPacket(BasePacket):
 
 class ResponsePacket(BasePacket):
 
-    __slots__ = ( 'ok', 'msg', 'data' )
+    __slots__ = ( 'ok', 'msg', 'data', 'in_response_to' )
 
-    def __init__(self, ok=True, msg=None, data=None):
+    def __init__(self, ok=True, msg=None, data=None, in_response_to=None):
 
         self.ok = ok
         self.msg = msg
         self.data = data
+        self.in_response_to = data
 
     def to_dict(self):
 
@@ -55,6 +62,8 @@ class ResponsePacket(BasePacket):
             data["data"] = self.data
         if self.msg is not None:
             data["msg"] = self.msg
+        if self.in_response_to is not None:
+            data["in_response_to"] = self.in_response_to
 
         return data
 
@@ -63,5 +72,6 @@ class ResponsePacket(BasePacket):
         return ResponsePacket(
             ok = data.get('ok', None),
             data = data.get('data', None),
-            msg = data.get('msg', None)
+            msg = data.get('msg', None),
+            in_response_to = data.get('in_response_to', None)
         )
